"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionAsync = actionAsync;
exports.default = _default;

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _clearNativeFolder() {
  const data = require("./eject/clearNativeFolder");

  _clearNativeFolder = function () {
    return data;
  };

  return data;
}

function _platformOptions() {
  const data = require("./eject/platformOptions");

  _platformOptions = function () {
    return data;
  };

  return data;
}

function _prebuildAsync() {
  const data = require("./eject/prebuildAsync");

  _prebuildAsync = function () {
    return data;
  };

  return data;
}

function _TerminalLink() {
  const data = require("./utils/TerminalLink");

  _TerminalLink = function () {
    return data;
  };

  return data;
}

function _maybeBailOnGitStatusAsync() {
  const data = _interopRequireDefault(require("./utils/maybeBailOnGitStatusAsync"));

  _maybeBailOnGitStatusAsync = function () {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function actionAsync(projectRoot, {
  platform,
  clean,
  skipDependencyUpdate,
  ...options
}) {
  if (options.npm) {
    options.packageManager = 'npm';
  }

  const platforms = (0, _platformOptions().platformsFromPlatform)(platform);

  if (clean) {
    if (await (0, _maybeBailOnGitStatusAsync().default)()) return; // Clear the native folders before syncing

    await (0, _clearNativeFolder().clearNativeFolder)(projectRoot, platforms);
  }

  await (0, _prebuildAsync().prebuildAsync)(projectRoot, { ...options,
    skipDependencyUpdate: skipDependencyUpdate ? skipDependencyUpdate.split(',') : [],
    platforms
  });
}

function _default(program) {
  program.command('prebuild [path]').description(`Experimental: Create native iOS and Android project files before building natively. ${_chalk().default.dim((0, _TerminalLink().learnMore)('https://docs.expo.io/workflow/customizing/'))}`).longDescription('Generate the native iOS and Android projects for your app before building them. The generated code should not be modified directly, instead config plugins should be used to make modifications.').helpGroup('eject').option('--no-install', 'Skip installing npm packages and CocoaPods.').option('--clean', 'Delete the native folders and regenerate them before applying changes').option('--npm', 'Use npm to install dependencies. (default when Yarn is not installed)').option('--template <template>', 'Project template to clone from. File path pointing to a local tar file or a github repo').option('-p, --platform [platform]', 'Platforms to sync: ios, android, all. Default: all').option('--skip-dependency-update <dependencies>', 'Preserves versions of listed packages in package.json (comma separated list)').asyncActionProjectDir(actionAsync);
}
//# sourceMappingURL=prebuild.js.map